@@ -5,15 +5,9 @@
 
 #include <QObject>
 #include <QQmlEngine>
-#include <QQuickItem>
 
 #include "neochatconnection.h"
 #include <Quotient/accountregistry.h>
-#include <Quotient/settings.h>
-
-#ifdef HAVE_KUNIFIEDPUSH
-#include <kunifiedpush/connector.h>
-#endif
 
 class TrayIcon;
 class QQuickTextDocument;
@@ -56,9 +50,19 @@ class Controller : public QObject
 
     Q_PROPERTY(QStringList accountsLoading MEMBER m_accountsLoading NOTIFY accountsLoadingChanged)
 
-    Q_PROPERTY(bool ssssSupported READ ssssSupported CONSTANT)
+    Q_PROPERTY(bool csSupported READ csSupported CONSTANT)
 
 public:
+    /**
+     * @brief Define the types on inline messages that can be shown.
+     */
+    enum MessageType {
+        Positive, /**< Positive message, typically green. */
+        Info, /**< Info message, typically highlight color. */
+        Error, /**< Error message, typically red. */
+    };
+    Q_ENUM(MessageType)
+
     static Controller &instance();
     static Controller *create(QQmlEngine *engine, QJSEngine *)
     {
@@ -82,7 +86,7 @@ public:
     /**
      * @brief Save an access token to the keychain for the given account.
      */
-    bool saveAccessTokenToKeyChain(const Quotient::AccountSettings &account, const QByteArray &accessToken);
+    bool saveAccessTokenToKeyChain(const QString &userId, const QByteArray &accessToken);
 
     [[nodiscard]] bool supportSystemTray() const;
 
@@ -102,7 +106,7 @@ public:
 
     Q_INVOKABLE void removeConnection(const QString &userId);
 
-    bool ssssSupported() const;
+    bool csSupported() const;
 
 private:
     explicit Controller(QObject *parent = nullptr);
@@ -110,7 +114,7 @@ private:
     QPointer<NeoChatConnection> m_connection;
     TrayIcon *m_trayIcon = nullptr;
 
-    QKeychain::ReadPasswordJob *loadAccessTokenFromKeyChain(const Quotient::AccountSettings &account);
+    QKeychain::ReadPasswordJob *loadAccessTokenFromKeyChain(const QString &account);
 
     void loadSettings();
     void saveSettings() const;
@@ -131,4 +135,5 @@ Q_SIGNALS:
     void connectionDropped(NeoChatConnection *connection);
     void activeConnectionChanged(NeoChatConnection *connection);
     void accountsLoadingChanged();
+    void showMessage(MessageType messageType, const QString &message);
 };
